@@ -26,46 +26,69 @@
  * IN THE SOFTWARE.
  */
 
-require("../conf/config.php");
-require("class.rFastTemplate.php");
+require(dirname(dirname(__FILE__))."/conf/config.php");
+require(dirname(__FILE__)."/class.rFastTemplate.php");
 
 function mlmmj_boolean($name, $nicename, $text)
 {
-    global $tpl, $topdir, $list, $HTTP_POST_VARS;
+    global $tpl, $topdir, $list;
     
     $file = $topdir."/".$list."/control/".$name;
     
-    if (isset($HTTP_POST_VARS[$name])) 
+    if(isset($_POST[$name]) && !empty($_POST[$name]))
     {
-	if(!touch($file))
-	    die("Couldn't open ".$file." for writing");
+       if(!touch($file))
+          die("Couldn't open ".$file." for writing");
+       if (!chmod($file, 0644))
+          die("Couldn't chmod ".$file);
+    }
+    else {
+       if (file_exists($file)) {
+          if (!unlink($file))
+             die("Couldn't unlink ".$file);
+       }
     }
-    else
-	@unlink($file);
 }
 
 function mlmmj_string ($name, $nicename, $text) 
 {
     mlmmj_list($name, $nicename, $text);
-}   
+}
 
 function mlmmj_list($name, $nicename, $text) 
 {
-    global $tpl, $topdir, $list,$HTTP_POST_VARS;
+    global $tpl, $topdir, $list;
 
     $file = $topdir."/".$list."/control/".$name;
     
-    if(!empty($HTTP_POST_VARS[$name]))
+    if(isset($_POST[$name]) && !empty($_POST[$name]) && !preg_match('/^\s*$/',$_POST[$name]))
     {
-	if (!$fp = fopen($file, "w"))
-	    die("Couldn't open ".$file." for writing");
+       // remove all \r
+       $_POST[$name]=preg_replace('/\r/',"",$_POST[$name]);
 
-	fwrite($fp, $HTTP_POST_VARS[$name]);
-	fclose($fp);
+       // no trailing \n?, then we add one
+       if (!preg_match('/\n$/',$_POST[$name]))
+          $_POST[$name].="\n";
+
+       // we don't like whitespace before a \n
+       $_POST[$name]=preg_replace('/\s*\n/',"\n",$_POST[$name]);
+
+       if (!$fp = fopen($file, "w"))
+          die("Couldn't open ".$file." for writing");
+
+       // write the result in a file
+       fwrite($fp, $_POST[$name]);
+       fclose($fp);
+
+       if (!chmod($file, 0644))
+          die("Couldn't chmod ".$file);
+    }
+    else {
+       if (file_exists($file)) {
+          if (!unlink($file))
+             die("Couldn't unlink ".$file);
+       }
     }
-    else
-	@unlink($file);
-    
 }
 
 // Perl's encode_entities (to be able to use tunables.pl)
@@ -74,21 +97,21 @@ function encode_entities($str) { return htmlentities($str); }
 
 $tpl = new rFastTemplate($templatedir);
 
-$list = $HTTP_POST_VARS["list"];
+$list = $_POST["list"];
 
 if(!isset($list))
 die("no list specified");
 
+if (dirname(realpath($topdir."/".$list)) != realpath($topdir))
+die("list outside topdir");
+
 if(!is_dir($topdir."/".$list))
 die("non-existent list");
 
 $tpl->define(array("main" => "save.html"));
 $tpl->assign(array("LIST" => htmlentities($list)));
 
-$handle = fopen("$templatedir/../conf/tunables.pl", "r");
-$tunables = fread($handle, filesize("$templatedir/../conf/tunables.pl"));
-fclose($handle);
-
+$tunables = file_get_contents($confdir.'/tunables.pl');
 eval($tunables);
 
 $tpl->parse("MAIN","main");
