@@ -1,6 +1,6 @@
 /* Copyright (C) 2002, 2003, 2004 Mads Martin Joergensen <mmj at mmj.dk>
  *
- * $Id: mlmmj.h,v 1.33 2007/06/12 18:28:58 mortenp Exp $
+ * $Id$
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -70,13 +70,30 @@ struct mailhdr {
 };
 
 /* Has to go here, since it's used in many places */
+
 enum subtype {
 	SUB_NORMAL,
 	SUB_DIGEST,
 	SUB_NOMAIL,
-	SUB_FILE /* For single files (moderator, owner etc.) */
+	SUB_FILE, /* For single files (moderator, owner etc.) */
+	SUB_ALL, /* For listing or unsubscribing all kinds of subscribers */
+	SUB_BOTH, /* For normal+digest subscription */
+	SUB_NONE /* For when an address is not subscribed at all */
 };
 
+char *subtype_strs[7]; /* count matches enum above; defined in subscriberfuncs.c */
+
+enum subreason {
+	SUB_REQUEST,
+	SUB_CONFIRM,
+	SUB_PERMIT,
+	SUB_ADMIN,
+	SUB_BOUNCING,
+	SUB_SWITCH
+};
+
+char * subreason_strs[6]; /* count matches enum above; defined in subscriberfuncs.c */
+
 void print_version(const char *prg);
 
 #define MY_ASSERT(expression) if (!(expression)) { \
