@@ -1,6 +1,6 @@
 /* Copyright (C) 2004 Mads Martin Joergensen <mmj at mmj.dk>
  *
- * $Id: mlmmj-list.c,v 1.4 2008/10/30 19:13:09 mortenp Exp $
+ * $Id$
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -39,8 +39,8 @@
 
 static void print_help(const char *prg)
 {
-        printf("Usage: %s -L /path/to/listdir [-c] [-d] [-h] [-m] [-n] "
-			"[-o] [-s] [-V]\n"
+	printf("Usage: %s -L /path/to/listdir\n"
+	       "       [-c] [-d] [-h] [-m] [-n] [-o] [-s] [-V]\n"
 	       " -L: Full path to list directory\n"
 	       " -c: Print subscriber count\n"
 	       " -d: Print for digesters list\n"
