@@ -137,8 +137,22 @@ int delolder(const char *dirname, time_t than)
 
 int clean_moderation(const char *listdir)
 {
-	char *moddirname = concatstr(2, listdir, "/moderation");
-	int ret = delolder(moddirname, MODREQLIFE);	
+
+	time_t modreqlife = 0;
+	char *modreqlifestr;
+	char *moddirname;
+	int ret;
+
+	modreqlifestr = ctrlvalue(listdir, "modreqlife");
+	if(modreqlifestr) {
+		modreqlife = atol(modreqlifestr);
+		myfree(modreqlifestr);
+	}
+	if(modreqlife == 0)
+		modreqlife = MODREQLIFE;
+
+	moddirname = concatstr(2, listdir, "/moderation");
+	ret = delolder(moddirname, modreqlife);
 		
 	myfree(moddirname);
 
