@@ -53,6 +53,7 @@
 #include "memory.h"
 #include "log_oper.h"
 #include "unistr.h"
+#include "chomp.h"
 
 enum action {
 	ALLOW,
@@ -74,6 +75,7 @@ static char *action_strs[] = {
 
 enum modreason {
 	MODNONSUBPOSTS,
+	MODNONMODPOSTS,
 	ACCESS,
 	MODERATED
 };
@@ -81,30 +83,16 @@ enum modreason {
 
 static char *modreason_strs[] = {
 	"modnonsubposts",
+	"modnonmodposts",
 	"access",
 	"moderated"
 };
 
 
-static void newmoderated(const char *listdir, const char *mailfilename,
-		  const char *mlmmjsend, const char *efromsender,
-		  const char *subject, const char *posteraddr,
-		  enum modreason modreason)
-{
-	char *from, *listfqdn, *listname, *moderators = NULL;
-	char *buf, *replyto, *listaddr = getlistaddr(listdir), *listdelim;
-	text *txt;
-	memory_lines_state *mls;
-	char *queuefilename = NULL, *moderatorsfilename, *efromismod = NULL;
-	char *mailbasename = mybasename(mailfilename), *tmp, *to, *reject;
-	int moderatorsfd, foundaddr = 0, notifymod = 0, status;
-	pid_t childpid, pid;
-#if 0
-	printf("mailfilename = [%s], mailbasename = [%s]\n", mailfilename,
-			                                     mailbasename);
-#endif
-	listfqdn = genlistfqdn(listaddr);
-	listname = genlistname(listaddr);
+static int is_moderator(const char *listdir, const char *address,
+		char **moderators) {
+	char *buf, *tmp, *moderatorsfilename;
+	int moderatorsfd, foundaddr = 0;
 
 	moderatorsfilename = concatstr(2, listdir, "/control/moderators");
 	if((moderatorsfd = open(moderatorsfilename, O_RDONLY)) < 0) {
@@ -114,27 +102,59 @@ static void newmoderated(const char *listdir, const char *mailfilename,
 	}
 	myfree(moderatorsfilename);
 
-	if(statctrl(listdir, "ifmodsendonlymodmoderate"))
-		efromismod = concatstr(2, efromsender, "\n");
-
 	while((buf = mygetline(moderatorsfd))) {
-		if(efromismod && strcmp(buf, efromismod) == 0)
+		chomp(buf);
+		if(address && strcasecmp(buf, address) == 0) {
 			foundaddr = 1;
-		tmp = moderators;
-		moderators = concatstr(2, moderators, buf);
+			if (!moderators) {
+				close(moderatorsfd);
+				myfree(buf);
+				return foundaddr;
+			}
+		}
+		if (moderators) {
+			tmp = *moderators;
+			*moderators = concatstr(3, *moderators, buf, "\n");
+			myfree(tmp);
+		}
 		myfree(buf);
-		myfree(tmp);
 	}
 
-	if(!foundaddr) {
-		myfree(efromismod);
+	close(moderatorsfd);
+	return foundaddr;
+}
+
+
+static void newmoderated(const char *listdir, const char *mailfilename,
+		  const char *mlmmjsend, const char *efromsender,
+		  const char *subject, const char *posteraddr,
+		  enum modreason modreason)
+{
+	char *from, *listfqdn, *listname, *moderators = NULL;
+	char *replyto, *listaddr = getlistaddr(listdir), *listdelim;
+	text *txt;
+	memory_lines_state *mls;
+	char *queuefilename = NULL;
+	const char *efromismod = NULL;
+	char *mailbasename = mybasename(mailfilename), *to, *reject;
+	int notifymod = 0, status;
+	pid_t childpid, pid;
+#if 0
+	printf("mailfilename = [%s], mailbasename = [%s]\n", mailfilename,
+			                                     mailbasename);
+#endif
+	listfqdn = genlistfqdn(listaddr);
+	listname = genlistname(listaddr);
+
+	if(statctrl(listdir, "ifmodsendonlymodmoderate"))
+		efromismod = efromsender;
+
+	if(!is_moderator(listdir, efromismod, &moderators))
 		efromismod = NULL;
-	}
 
 	if(efromismod) mls = init_memory_lines(efromismod);
 	else mls = init_memory_lines(moderators);
 
-	close(moderatorsfd);
 	myfree(moderators);
 
 	listdelim = getlistdelim(listdir);
@@ -368,35 +388,51 @@ static enum action do_access(struct strlist *rule_strs, struct strlist *hdrs,
 }
 
 
-static char *recipient_extra(const char *listdir, const char *addr)
+static int addrmatch(const char *listaddr, const char *addr,
+		const char *listdelim, char **recipextra)
 {
-	char *listdelim;
-	char *delim, *atsign, *ret;
+	char *delim, *atsign;
 	size_t len;
 
 	if (!addr)
-		return NULL;
+		return 0;
 
-	listdelim = getlistdelim(listdir);
+	if(strcasecmp(listaddr, addr) == 0) {
+		if (recipextra)
+			*recipextra = NULL;
+		return 1;
+	}
+
+	if (!listdelim)
+		return 0;
 
 	delim = strstr(addr, listdelim);
-	if (!delim) {
-		myfree(listdelim);
-		return NULL;
-	}
+	if (!delim)
+		return 0;
+
+	len = delim - addr;
+	if(strncasecmp(listaddr, addr, len) != 0)
+		return 0;
+	if(*(listaddr + len) != '@')
+		return 0;
+
 	delim += strlen(listdelim);
-	myfree(listdelim);
 
 	atsign = strrchr(delim, '@');
 	if (!atsign)
-		return NULL;
+		return 0;
 
-	len = atsign - delim;
-	ret = (char *)mymalloc(len + 1);
-	strncpy(ret, delim, len);
-	ret[len] = '\0';
+	if(strcasecmp(listaddr + len + 1, atsign + 1) != 0)
+		return 0;
 
-	return ret;
+	if (recipextra) {
+		len = atsign - delim;
+		*recipextra = (char *)mymalloc(len + 1);
+		strncpy(*recipextra, delim, len);
+		(*recipextra)[len] = '\0';
+	}
+
+	return 1;
 }
 
 
@@ -418,14 +454,15 @@ int main(int argc, char **argv)
 	int i, j, opt, noprocess = 0, moderated = 0, send = 0;
 	enum modreason modreason;
 	int hdrfd, footfd, rawmailfd, donemailfd, omitfd;
-	int addrtocc = 1, intocc = 0;
+	int addrtocc, intocc = 0, findaddress = 0;
 	int maxmailsize = 0;
 	int notmetoo = 0;
+	int subonlypost = 0, modonlypost = 0, modnonsubposts = 0, foundaddr = 0;
 	char *listdir = NULL, *mailfile = NULL, *headerfilename = NULL;
 	char *footerfilename = NULL, *donemailname = NULL;
 	char *randomstr = NULL, *mqueuename, *omitfilename;
 	char *mlmmjsend, *mlmmjsub, *mlmmjunsub, *mlmmjbounce;
-	char *bindir, *subjectprefix, *discardname, *listaddr, *listdelim;
+	char *bindir, *subjectprefix, *discardname, *listaddr, *listdelim = NULL;
 	char *listfqdn, *listname, *fromaddr;
 	text *txt;
 	char *queuefilename, *recipextra = NULL, *owner = NULL;
@@ -442,7 +479,7 @@ int main(int argc, char **argv)
 	struct strlist *access_rules = NULL;
 	struct strlist *delheaders = NULL;
 	struct strlist allheaders;
-	struct strlist *alternates = NULL;
+	struct strlist *listaddrs = NULL;
 	struct mailhdr readhdrs[] = {
 		{ "From:", 0, NULL },
 		{ "To:", 0, NULL },
@@ -610,15 +647,57 @@ int main(int argc, char **argv)
 			recipextra = mystrdup(envstr);
 		}
 		myfree(listdelim);
-	} else if(dtemails.emailcount >= 1) {
-		/* parse the (first) Delivered-To: header */
-		recipextra = recipient_extra(listdir, dtemails.emaillist[0]);
-	} else if(toemails.emailcount >= 1) {
-		/* parse the (first) To: header */
-		recipextra = recipient_extra(listdir, toemails.emaillist[0]);
+		listdelim = NULL;
 	} else {
-		recipextra = NULL;
+		findaddress = 1;
 	}
+
+	addrtocc = !(statctrl(listdir, "tocc"));
+
+	if (findaddress) {
+		listdelim = getlistdelim(listdir);
+	}
+	if(addrtocc || findaddress) {
+		listaddrs = ctrlvalues(listdir, "listaddress");
+		for(i = 0; i < dtemails.emailcount; i++) {
+			for(j = 0; j < listaddrs->count; j++) {
+				if(addrmatch(listaddrs->strs[j], dtemails.emaillist[i],
+						listdelim, &recipextra)) {
+					findaddress = 0;
+					myfree(listdelim);
+					listdelim = NULL;
+					break;
+				}
+			}
+		}
+	}
+	if(addrtocc || findaddress) {
+		for(i = 0; i < toemails.emailcount; i++) {
+			for(j = 0; j < listaddrs->count; j++) {
+				if(addrmatch(listaddrs->strs[j], toemails.emaillist[i],
+						listdelim, &recipextra)) {
+					intocc = 1;
+					break;
+				}
+			}
+		}
+		if (!intocc) for(i = 0; i < ccemails.emailcount; i++) {
+			for(j = 0; j < listaddrs->count; j++) {
+				if(addrmatch(listaddrs->strs[j], ccemails.emaillist[i],
+						listdelim, &recipextra)) {
+					intocc = 1;
+					break;
+				}
+			}
+		}
+	}
+	if (listdelim) {
+		myfree(listdelim);
+		listdelim = NULL;
+	}
+	if (listaddrs) for(i = 0; i < listaddrs->count; i++)
+		myfree(listaddrs->strs[i]);
+
 	if(recipextra && (strlen(recipextra) == 0)) {
 		myfree(recipextra);
 		recipextra = NULL;
@@ -729,7 +808,7 @@ int main(int argc, char **argv)
 			exit(EXIT_FAILURE);
 		}
 #if 0
-		log_error(LOG_ARGS, "listcontrol(from, %s, %s, %s, %s, %s, %s)\n", listdir, toemails.emaillist[0], mlmmjsub, mlmmjunsub, mlmmjsend, mlmmjbounce);
+		log_error(LOG_ARGS, "listcontrol(from, %s, %s, %s, %s, %s, %s, %s)\n", listdir, toemails.emaillist[0], mlmmjsub, mlmmjunsub, mlmmjsend, mlmmjbounce, donemailname);
 #endif
 		unlink(mailfile);
 		listcontrol(&fromemails, listdir, recipextra,
@@ -740,7 +819,6 @@ int main(int argc, char **argv)
 	}
 
 	listaddr = getlistaddr(listdir);
-	alternates = ctrlvalues(listdir, "listaddress");
 
 	/* checking incoming mail's size */
 	maxmailsizestr = ctrlvalue(listdir, "maxmailsize");
@@ -821,33 +899,6 @@ int main(int argc, char **argv)
 
 	unlink(mailfile);
 
-	addrtocc = !(statctrl(listdir, "tocc"));
-	if(addrtocc) {
-		for(i = 0; i < toemails.emailcount; i++) {
-			errno = 0;
-			log_error(LOG_ARGS, "Found To: %s",
-				toemails.emaillist[i]);
-			for(j = 0; j < alternates->count; j++) {
-				if(strcasecmp(alternates->strs[j],
-					toemails.emaillist[i]) == 0)
-					intocc = 1;
-			}
-		}
-		for(i = 0; i < ccemails.emailcount; i++) {
-			errno = 0;
-			log_error(LOG_ARGS, "Found Cc: %s",
-				ccemails.emaillist[i]);
-			for(j = 0; j < alternates->count; j++) {
-				if(strcasecmp(alternates->strs[j],
-					ccemails.emaillist[i]) == 0)
-					intocc = 1;
-			}
-		}
-	}
-
-	for(i = 0; i < alternates->count; i++)
-		myfree(alternates->strs[i]);
-
 	if(addrtocc && !intocc) {
 		/* Don't send a mail about denial to the list, but silently
 		 * discard and exit. Also don't in case of it being turned off
@@ -972,65 +1023,86 @@ int main(int argc, char **argv)
 		}
 	}
 
-	if(!send && (statctrl(listdir, "subonlypost") ||
-			statctrl(listdir, "modnonsubposts"))) {
+	subonlypost = statctrl(listdir, "subonlypost");
+	modonlypost = statctrl(listdir, "modonlypost");
+	modnonsubposts = statctrl(listdir, "modnonsubposts");
+	/* modnonsubposts implies subonlypost if modonlypost is not set */
+	if (modnonsubposts && !modonlypost) subonlypost = 1;
+
+	if(!send && (subonlypost || modonlypost || modnonsubposts)) {
 		/* Don't send a mail about denial to the list, but silently
 		 * discard and exit. */
 		if (strcasecmp(listaddr, posteraddr) == 0) {
 			log_error(LOG_ARGS, "Discarding %s because"
-					" subonlypost was set and From: was"
-					" the list address",
+					" there are sender restrictions but"
+					" From: was the list address",
 					mailfile);
 			myfree(listaddr);
 			unlink(donemailname);
 			myfree(donemailname);
 			exit(EXIT_SUCCESS);
 		}
-		if(is_subbed(listdir, posteraddr, 0) == SUB_NONE) {
-			if(statctrl(listdir, "modnonsubposts")) {
-				moderated = 1;
+		if(subonlypost) {
+			foundaddr = (is_subbed(listdir, posteraddr, 0) !=
+					SUB_NONE);
+		} else if (modonlypost) {
+			foundaddr = is_moderator(listdir, posteraddr, NULL);
+		}
+		if(!foundaddr) {
+			if(modnonsubposts) {
+			    moderated = 1;
+			    if (subonlypost)
 				modreason = MODNONSUBPOSTS;
+			    else if (modonlypost)
+				modreason = MODNONMODPOSTS;
 			} else {
-				if(statctrl(listdir, "nosubonlydenymails")) {
-				    log_error(LOG_ARGS, "Discarding %s because"
-					    " subonlypost and"
-					    " nosubonlydenymails was set",
-					    mailfile);
-				    myfree(listaddr);
-				    unlink(donemailname);
-				    myfree(donemailname);
-				    exit(EXIT_SUCCESS);
-				}
-				listdelim = getlistdelim(listdir);
-				listname = genlistname(listaddr);
-				listfqdn = genlistfqdn(listaddr);
-				fromaddr = concatstr(4, listname, listdelim,
-					"bounces-help@", listfqdn);
-				txt = open_text(listdir, "deny", "post",
-					"subonlypost", NULL, "subonlypost");
-				MY_ASSERT(txt);
-				register_unformatted(txt, "subject", subject);
-				register_unformatted(txt, "posteraddr", posteraddr);
-				register_originalmail(txt, donemailname);
-				queuefilename = prepstdreply(txt, listdir,
-					"$listowner$", posteraddr, NULL);
-				MY_ASSERT(queuefilename)
-				close_text(txt);
+			    if((subonlypost &&
+				    statctrl(listdir, "nosubonlydenymails")) ||
+				    (modonlypost &&
+				    statctrl(listdir, "nomodonlydenymails"))) {
+				log_error(LOG_ARGS, "Discarding %s because"
+					" no{sub|mod}onlydenymails was set",
+					mailfile);
 				myfree(listaddr);
-				myfree(listdelim);
-				myfree(listname);
-				myfree(listfqdn);
 				unlink(donemailname);
 				myfree(donemailname);
-				execlp(mlmmjsend, mlmmjsend,
-					"-L", listdir,
-					"-l", "1",
-					"-T", posteraddr,
-					"-F", fromaddr,
-					"-m", queuefilename, (char *)NULL);
+				exit(EXIT_SUCCESS);
+			    }
+			    listdelim = getlistdelim(listdir);
+			    listname = genlistname(listaddr);
+			    listfqdn = genlistfqdn(listaddr);
+			    fromaddr = concatstr(4, listname, listdelim,
+				    "bounces-help@", listfqdn);
+			    if (subonlypost) {
+				txt = open_text(listdir, "deny", "post",
+					"subonlypost", NULL, "subonlypost");
+			    } else if (modonlypost) {
+				txt = open_text(listdir, "deny", "post",
+					"modonlypost", NULL, NULL);
+			    }
+			    MY_ASSERT(txt);
+			    register_unformatted(txt, "subject", subject);
+			    register_unformatted(txt, "posteraddr", posteraddr);
+			    register_originalmail(txt, donemailname);
+			    queuefilename = prepstdreply(txt, listdir,
+				    "$listowner$", posteraddr, NULL);
+			    MY_ASSERT(queuefilename)
+			    close_text(txt);
+			    myfree(listaddr);
+			    myfree(listdelim);
+			    myfree(listname);
+			    myfree(listfqdn);
+			    unlink(donemailname);
+			    myfree(donemailname);
+			    execlp(mlmmjsend, mlmmjsend,
+				    "-L", listdir,
+				    "-l", "1",
+				    "-T", posteraddr,
+				    "-F", fromaddr,
+				    "-m", queuefilename, (char *)NULL);
 
-				log_error(LOG_ARGS, "execlp() of '%s' failed", mlmmjsend);
-				exit(EXIT_FAILURE);
+			    log_error(LOG_ARGS, "execlp() of '%s' failed", mlmmjsend);
+			    exit(EXIT_FAILURE);
 			}
 		}
 	}
