@@ -1,6 +1,6 @@
 /* Copyright (C) 2002, 2003 Mads Martin Joergensen <mmj at mmj.dk>
  *
- * $Id: mlmmj-sub.c,v 1.54 2007/11/05 21:23:37 mortenp Exp $
+ * $Id$
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -50,18 +50,18 @@
 #include "ctrlvalues.h"
 #include "chomp.h"
 
-void moderate_sub(const char *listdir, const char *listaddr,
+static void moderate_sub(const char *listdir, const char *listaddr,
 		const char *listdelim, const char *subaddr,
-		const char *mlmmjsend, enum subtype typesub)
+		const char *mlmmjsend, enum subtype typesub, enum subreason reasonsub)
 {
-	int i, fd;
+	int i, fd, status, nosubmodmails = 0;
+	text *txt;
+	memory_lines_state *mls;
 	char *a = NULL, *queuefilename, *from, *listname, *listfqdn, *str;
-	char *modfilename, *randomstr, *mods, *to, *replyto, *moderators = NULL;
-	char *modfilebase;
+	char *modfilename, *mods, *to, *replyto, *moderators = NULL;
+	char *cookie, *obstruct;
 	struct strlist *submods;
-	pid_t pid;
-	char *maildata[6] = { "subaddr", NULL, "moderateaddr", NULL,
-				"moderators", NULL };
+	pid_t childpid, pid;
 
 	/* generate the file in moderation/ */
 	switch(typesub) {
@@ -75,27 +75,29 @@ void moderate_sub(const char *listdir, const char *listaddr,
 		case SUB_NOMAIL:
 			str = concatstr(4, subaddr, "\n", "SUB_NOMAIL", "\n");
 			break;
+		case SUB_BOTH:
+			str = concatstr(4, subaddr, "\n", "SUB_BOTH", "\n");
+			break;
 	}
 	
-	randomstr = random_str();
-	modfilename = concatstr(3, listdir, "/moderation/subscribe",
-			randomstr);
-	myfree(randomstr);
-
-	fd = open(modfilename, O_RDWR|O_CREAT|O_EXCL, S_IRUSR|S_IWUSR);
-	while(fd < 0 && errno == EEXIST) {
-		myfree(modfilename);
-		randomstr = random_str();
+	for (;;) {
+		cookie = random_str();
 		modfilename = concatstr(3, listdir, "/moderation/subscribe",
-				randomstr);
-		myfree(randomstr);
+				cookie);
 		fd = open(modfilename, O_RDWR|O_CREAT|O_EXCL, S_IRUSR|S_IWUSR);
+		if (fd < 0) {
+			if (errno == EEXIST) {
+				myfree(cookie);
+				myfree(modfilename);
+				continue;
+			}
+			log_error(LOG_ARGS, "could not create %s"
+					"ignoring request: %s", str);
+			exit(EXIT_FAILURE);
+		}
+		break;
 	}
-	if(fd < 0) {
-		log_error(LOG_ARGS, "could not create %s"
-				"ignoring request: %s", str);
-		exit(EXIT_FAILURE);
-	}
+
 	if(writen(fd, str, strlen(str)) < 0) {
 		log_error(LOG_ARGS, "could not write to %s"
 				"ignoring request: %s", str);
@@ -128,39 +130,58 @@ void moderate_sub(const char *listdir, const char *listaddr,
 	listdelim = getlistdelim(listdir);
 	listfqdn = genlistfqdn(listaddr);
 	listname = genlistname(listaddr);
-	modfilebase = mybasename(modfilename);
 
 	from = concatstr(4, listname, listdelim, "owner@", listfqdn);
 	to = concatstr(3, listname, "-moderators@", listfqdn);
-	replyto = concatstr(6, listname, listdelim, "moderate-", modfilebase,
+	replyto = concatstr(6, listname, listdelim, "permit-", cookie,
 			"@", listfqdn);
-	myfree(modfilebase);
+	obstruct = concatstr(6, listname, listdelim, "obstruct-", cookie,
+			"@", listfqdn);
+	myfree(cookie);
 	for(i = 0; i < submods->count; i++) {
 		printf("%s", submods->strs[i]);
 		str = moderators;
 		moderators = concatstr(3, moderators, submods->strs[i], "\n");
 		myfree(str);
 	}
+	mls = init_memory_lines(moderators);
+	myfree(moderators);
 
-	maildata[1] = mystrdup(subaddr);
-	maildata[3] = replyto;
-	maildata[5] = moderators;
-
-	queuefilename = prepstdreply(listdir, "submod-moderator",
-				"$listowner$", to, replyto, 3, maildata, NULL, NULL);
+	txt = open_text(listdir,
+			"gatekeep", "sub",
+			subreason_strs[reasonsub], subtype_strs[typesub],
+			"submod-moderator");
+	MY_ASSERT(txt);
+	register_unformatted(txt, "subaddr", subaddr);
+	register_unformatted(txt, "moderateaddr", replyto); /* DEPRECATED */
+	register_unformatted(txt, "permitaddr", replyto);
+	register_unformatted(txt, "obstructaddr", obstruct);
+	register_unformatted(txt, "moderators", "%gatekeepers%"); /* DEPRECATED */
+	register_formatted(txt, "gatekeepers",
+			rewind_memory_lines, get_memory_line, mls);
+	queuefilename = prepstdreply(txt, listdir, "$listowner$", to, replyto);
+	MY_ASSERT(queuefilename);
+	close_text(txt);
 	
-	myfree(maildata[1]);
+	/* we might need to exec more than one mlmmj-send */
 	
-	/* we need to exec more than one mlmmj-send */
+	nosubmodmails = statctrl(listdir,"nosubmodmails");
 	
-	pid = fork();
-
-	if(pid < 0) {
-		log_error(LOG_ARGS, "Could not fork for mlmmj-send"
-				"Request in %s not served", modfilename);
-		exit(EXIT_FAILURE);
+	if (nosubmodmails)
+		childpid = -1;
+	else {
+		childpid = fork();
+		if(childpid < 0)
+			log_error(LOG_ARGS, "Could not fork; requester not notified");
 	}
-	if(pid == 0) {
+
+	if(childpid != 0) {
+		if(childpid > 0) {
+			do /* Parent waits for the child */
+				pid = waitpid(childpid, &status, 0);
+			while(pid == -1 && errno == EINTR);
+		}
+		finish_memory_lines(mls);
 		execl(mlmmjsend, mlmmjsend,
 				"-a",
 				"-l", "4",
@@ -172,17 +193,29 @@ void moderate_sub(const char *listdir, const char *listaddr,
 		log_error(LOG_ARGS, "execl() of '%s' failed", mlmmjsend);
 		exit(EXIT_FAILURE);
 	}
-	
+
 	myfree(to);
 	myfree(replyto);
-	myfree(moderators);
 	
 	/* send mail to requester that the list is submod'ed */
 
 	from = concatstr(4, listname, listdelim, "bounces-help@", listfqdn);
-	queuefilename = prepstdreply(listdir, "submod-requester", "$listowner$",
-					subaddr, NULL, 0, NULL, NULL, NULL);
-	
+
+	txt = open_text(listdir,
+			"wait", "sub",
+			subreason_strs[reasonsub], subtype_strs[typesub],
+			"submod-requester");
+	MY_ASSERT(txt);
+	register_unformatted(txt, "subaddr", subaddr);
+	register_unformatted(txt, "moderators", "%gatekeepers"); /* DEPRECATED */
+	register_formatted(txt, "gatekeepers",
+			rewind_memory_lines, get_memory_line, mls);
+	queuefilename = prepstdreply(txt, listdir,
+			"$listowner$", subaddr, NULL);
+	MY_ASSERT(queuefilename);
+	close_text(txt);
+
+	finish_memory_lines(mls);
 	myfree(listname);
 	myfree(listfqdn);
 	execl(mlmmjsend, mlmmjsend,
@@ -200,8 +233,11 @@ void getaddrandtype(const char *listdir, const char *modstr,
 {
 	int fd;
 	char *readaddr, *readtype, *modfilename;
-		
-	modfilename = concatstr(3, listdir, "/moderation/", modstr);
+
+	if (strncmp(modstr, "subscribe", 9) == 0)
+			modstr += 9;
+
+	modfilename = concatstr(3, listdir, "/moderation/subscribe", modstr);
 
 	fd = open(modfilename, O_RDONLY);
 	if(fd < 0) {
@@ -237,6 +273,11 @@ void getaddrandtype(const char *listdir, const char *modstr,
 		goto freedone;
 	}
 
+	if(strncmp(readtype, "SUB_BOTH", 8) == 0) {
+		*subtypeptr = SUB_BOTH;
+		goto freedone;
+	}
+
 	log_error(LOG_ARGS, "Type %s not valid in %s", readtype,
 			modfilename);
 
@@ -248,8 +289,9 @@ freedone:
 
 void confirm_sub(const char *listdir, const char *listaddr,
 		const char *listdelim, const char *subaddr,
-		const char *mlmmjsend, enum subtype typesub)
+		const char *mlmmjsend, enum subtype typesub, enum subreason reasonsub)
 {
+	text *txt;
 	char *queuefilename, *fromaddr, *listname, *listfqdn, *listtext;
 
 	listname = genlistname(listaddr);
@@ -271,12 +313,22 @@ void confirm_sub(const char *listdir, const char *listaddr,
 		case SUB_NOMAIL:
 			listtext = mystrdup("sub-ok-nomail");
 			break;
+		case SUB_BOTH:
+			/* No legacy list text as feature didn't exist. */
+			listtext = mystrdup("sub-ok");
+			break;
 	}
 
-	queuefilename = prepstdreply(listdir, listtext, "$helpaddr$",
-				     subaddr, NULL, 0, NULL, NULL, NULL);
-	MY_ASSERT(queuefilename);
+	txt = open_text(listdir, "finish", "sub",
+			subreason_strs[reasonsub], subtype_strs[typesub],
+			listtext);
 	myfree(listtext);
+	MY_ASSERT(txt);
+	register_unformatted(txt, "subaddr", subaddr);
+	queuefilename = prepstdreply(txt, listdir,
+			"$helpaddr$", subaddr, NULL);
+	MY_ASSERT(queuefilename);
+	close_text(txt);
 
 	execlp(mlmmjsend, mlmmjsend,
 				"-l", "1",
@@ -290,17 +342,15 @@ void confirm_sub(const char *listdir, const char *listaddr,
 
 void notify_sub(const char *listdir, const char *listaddr,
 		const char *listdelim, const char *subaddr,
-		const char *mlmmjsend, enum subtype typesub)
+		const char *mlmmjsend, enum subtype typesub, enum subreason reasonsub)
 {
-	char *maildata[2] = { "newsub", NULL };
 	char *listfqdn, *listname, *fromaddr, *tostr;
+	text *txt;
 	char *queuefilename = NULL, *listtext = NULL;
 
 	listname = genlistname(listaddr);
 	listfqdn = genlistfqdn(listaddr);
 
-	maildata[1] = mystrdup(subaddr);
-	
 	fromaddr = concatstr(4, listname, listdelim, "bounces-help@", listfqdn);
 	tostr = concatstr(4, listname, listdelim, "owner@", listfqdn);
 	
@@ -318,13 +368,24 @@ void notify_sub(const char *listdir, const char *listaddr,
 		case SUB_NOMAIL:
 			listtext = mystrdup("notifysub-nomail");
 			break;
+		case SUB_BOTH:
+			/* No legacy list text as feature didn't exist. */
+			listtext = mystrdup("notifysub");
+			break;
 	}
 
-	queuefilename = prepstdreply(listdir, listtext, "$listowner$",
-				"$listowner$", NULL, 1, maildata, NULL, NULL);
-	MY_ASSERT(queuefilename)
+	txt = open_text(listdir, "notify", "sub",
+			subreason_strs[reasonsub], subtype_strs[typesub],
+			listtext);
 	myfree(listtext);
-	myfree(maildata[1]);
+	MY_ASSERT(txt);
+	register_unformatted(txt, "subaddr", subaddr);
+	register_unformatted(txt, "newsub", subaddr); /* DEPRECATED */
+	queuefilename = prepstdreply(txt, listdir,
+			"$listowner$", "$listowner$", NULL);
+	MY_ASSERT(queuefilename);
+	close_text(txt);
+
 	execlp(mlmmjsend, mlmmjsend,
 			"-l", "1",
 			"-L", listdir,
@@ -338,13 +399,13 @@ void notify_sub(const char *listdir, const char *listaddr,
 
 void generate_subconfirm(const char *listdir, const char *listaddr,
 			 const char *listdelim, const char *subaddr,
-			 const char *mlmmjsend, enum subtype typesub)
+			 const char *mlmmjsend, enum subtype typesub, enum subreason reasonsub)
 {
 	int subconffd;
 	char *confirmaddr, *listname, *listfqdn, *confirmfilename = NULL;
+	text *txt;
 	char *listtext, *queuefilename = NULL, *fromaddr;
 	char *randomstr = NULL, *tmpstr;
-	char *maildata[4] = { "subaddr", NULL, "confaddr", NULL };
 
 	listname = genlistname(listaddr);
 	listfqdn = genlistfqdn(listaddr);
@@ -396,6 +457,11 @@ void generate_subconfirm(const char *listdir, const char *listaddr,
 			listtext = mystrdup("sub-confirm-nomail");
 			tmpstr = mystrdup("confsub-nomail-");
 			break;
+		case SUB_BOTH:
+			/* No legacy list text as feature didn't exist. */
+			listtext = mystrdup("sub-confirm");
+			tmpstr = mystrdup("confsub-both-");
+			break;
 	}
 
 	confirmaddr = concatstr(6, listname, listdelim, tmpstr, randomstr, "@",
@@ -404,14 +470,18 @@ void generate_subconfirm(const char *listdir, const char *listaddr,
 	myfree(randomstr);
 	myfree(tmpstr);
 
-	maildata[1] = mystrdup(subaddr);
-	maildata[3] = mystrdup(confirmaddr);
-
-	queuefilename = prepstdreply(listdir, listtext, "$helpaddr$", subaddr,
-				     confirmaddr, 2, maildata, NULL, NULL);
-
-	myfree(maildata[1]);
-	myfree(maildata[3]);
+	txt = open_text(listdir, "confirm", "sub",
+			subreason_strs[reasonsub], subtype_strs[typesub],
+			listtext);
+	myfree(listtext);
+	MY_ASSERT(txt);
+	register_unformatted(txt, "subaddr", subaddr);
+	register_unformatted(txt, "confaddr", confirmaddr); /* DEPRECATED */
+	register_unformatted(txt, "confirmaddr", confirmaddr);
+	queuefilename = prepstdreply(txt, listdir,
+			"$helpaddr$", subaddr, confirmaddr);
+	MY_ASSERT(queuefilename);
+	close_text(txt);
 
 	myfree(listname);
 	myfree(listfqdn);
@@ -428,27 +498,31 @@ void generate_subconfirm(const char *listdir, const char *listaddr,
 
 static void print_help(const char *prg)
 {
-	printf("Usage: %s -L /path/to/list [-a john@doe.org | -m str]"
-	       "[-c] [-C] [-h]\n       [-L] [-d | -n] [-s] [-U] [-V]\n"
+	printf("Usage: %s -L /path/to/list {-a john@doe.org | -m str}\n"
+	       "       [-c] [-C] [-f] [-h] [-L] [-d | -n] [-q] [-r | -R] [-s] [-U] [-V]\n"
 	       " -a: Email address to subscribe \n"
-	       " -c: Send welcome mail\n"
-	       " -C: Request mail confirmation\n"
+	       " -c: Send welcome mail (unless requesting confirmation)\n"
+	       " -C: Request mail confirmation (unless switching versions)\n"
 	       " -d: Subscribe to digest of list\n"
+	       " -f: Force subscription (do not moderate)\n"
 	       " -h: This help\n"
 	       " -L: Full path to list directory\n"
 	       " -m: moderation string\n"
 	       " -n: Subscribe to no mail version of list\n", prg);
-	printf(" -s: Don't send a mail to subscriber if already subscribed\n"
+	printf(" -q: Be quiet (don't notify owner about the subscription)\n"
+	       " -r: Behave as if request arrived via email (internal use)\n"
+	       " -R: Behave as if confirmation arrived via email (internal use)\n"
+	       " -s: Don't send a mail to subscriber if already subscribed\n"
 	       " -U: Don't switch to the user id of the listdir owner\n"
 	       " -V: Print version\n"
-	       "When no options are specified, subscription silently "
-	       "happens\n");
+	       "To ensure a silent subscription, use -f -q -s\n");
 	exit(EXIT_SUCCESS);
 }
 
 void generate_subscribed(const char *listdir, const char *subaddr,
-		const char *mlmmjsend)
+		const char *mlmmjsend, enum subtype typesub)
 {
+	text *txt;
 	char *queuefilename, *fromaddr, *listname, *listfqdn, *listaddr;
 	char *listdelim = getlistdelim(listdir);
 
@@ -459,9 +533,15 @@ void generate_subscribed(const char *listdir, const char *subaddr,
 	fromaddr = concatstr(4, listname, listdelim, "bounces-help@", listfqdn);
 	myfree(listdelim);
 
-	queuefilename = prepstdreply(listdir, "sub-subscribed", "$helpaddr$",
-				     subaddr, NULL, 0, NULL, NULL, NULL);
+	txt = open_text(listdir,
+			"deny", "sub", "subbed", subtype_strs[typesub],
+			"sub-subscribed");
+	MY_ASSERT(txt);
+	register_unformatted(txt, "subaddr", subaddr);
+	queuefilename = prepstdreply(txt, listdir,
+			"$helpaddr$", subaddr, NULL);
 	MY_ASSERT(queuefilename);
+	close_text(txt);
 
 	myfree(listaddr);
 	myfree(listname);
@@ -478,120 +558,15 @@ void generate_subscribed(const char *listdir, const char *subaddr,
 	exit(EXIT_FAILURE);
 }
 
-int main(int argc, char **argv)
-{
-	char *listaddr, *listdelim, *listdir = NULL, *address = NULL;
-	char *subfilename = NULL, *mlmmjsend, *bindir, chstr[2], *subdir;
-	char *subddirname = NULL, *sublockname, *lowcaseaddr;
-	char *modstr = NULL;
-	int subconfirm = 0, confirmsub = 0, opt, subfilefd, lock, notifysub;
-	int changeuid = 1, status, digest = 0, nomail = 0, i = 0, submod = 0;
-	int groupwritable = 0, sublock, sublockfd, nogensubscribed = 0, subbed;
-	size_t len;
+static void subscribe_type(char *listdir, char *listaddr, char *listdelim,
+		char *address, char *mlmmjsend,
+		enum subtype typesub, enum subreason reasonsub) {
+	char *subfilename = NULL;
+	char chstr[2], *subdir;
+	char *subddirname = NULL, *sublockname;
+	int groupwritable = 0, sublock, sublockfd, lock, subfilefd;
 	struct stat st;
-	pid_t pid, childpid;
-	uid_t uid;
-	enum subtype typesub = SUB_NORMAL;
-
-	CHECKFULLPATH(argv[0]);
-
-	log_set_name(argv[0]);
-
-	bindir = mydirname(argv[0]);
-	mlmmjsend = concatstr(2, bindir, "/mlmmj-send");
-	myfree(bindir);
-
-	while ((opt = getopt(argc, argv, "hcCdm:nsVUL:a:")) != -1) {
-		switch(opt) {
-		case 'a':
-			address = optarg;
-			break;
-		case 'c':
-			confirmsub = 1;
-			break;
-		case 'C':
-			subconfirm = 1;
-			break;
-		case 'd':
-			digest = 1;
-			break;
-		case 'h':
-			print_help(argv[0]);
-			break;
-		case 'L':
-			listdir = optarg;
-			break;
-		case 'm':
-			modstr = optarg;
-			break;
-		case 'n':
-			nomail = 1;
-			break;
-		case 's':
-			nogensubscribed = 1;
-			break;
-		case 'U':
-			changeuid = 0;
-			break;
-		case 'V':
-			print_version(argv[0]);
-			exit(0);
-		}
-	}
-
-	if(listdir == NULL) {
-		fprintf(stderr, "You have to specify -L\n");
-		fprintf(stderr, "%s -h for help\n", argv[0]);
-		exit(EXIT_FAILURE);
-	}
-	
-	if(address == NULL && modstr == NULL) {
-		fprintf(stderr, "You have to specify -a or -m\n");
-		fprintf(stderr, "%s -h for help\n", argv[0]);
-		exit(EXIT_FAILURE);
-	}
-
-	if(modstr)
-		getaddrandtype(listdir, modstr, &address, &typesub);
-
-	if(strchr(address, '@') == NULL) {
-		log_error(LOG_ARGS, "No '@' sign in '%s', not subscribing",
-				address);
-		exit(EXIT_SUCCESS);
-	}
-
-	if(digest && nomail) {
-		fprintf(stderr, "Specify either -d or -n, not both\n");
-		fprintf(stderr, "%s -h for help\n", argv[0]);
-		exit(EXIT_FAILURE);
-	}
-
-	if(digest)
-		typesub = SUB_DIGEST;
-	if(nomail)
-		typesub = SUB_NOMAIL;
-
-	if(confirmsub && subconfirm) {
-		fprintf(stderr, "Cannot specify both -C and -c\n");
-		fprintf(stderr, "%s -h for help\n", argv[0]);
-		exit(EXIT_FAILURE);
-	}
-
-	/* Make the address lowercase */
-	lowcaseaddr = mystrdup(address);
-	i = 0;
-	while(lowcaseaddr[i]) {
-		lowcaseaddr[i] = tolower(lowcaseaddr[i]);
-		i++;
-	}
-	address = lowcaseaddr;
-
-	/* get the list address */
-	listaddr = getlistaddr(listdir);
-	if(strncasecmp(listaddr, address, strlen(listaddr)) == 0) {
-		printf("Cannot subscribe the list address to the list\n");
-		exit(EXIT_SUCCESS);  /* XXX is this success? */
-	}
+	size_t len;
 
 	switch(typesub) {
 		default:
@@ -615,19 +590,6 @@ int main(int argc, char **argv)
 		}
 	}
 
-	if(changeuid) {
-		uid = getuid();
-		if(!uid && stat(listdir, &st) == 0) {
-			printf("Changing to uid %d, owner of %s.\n",
-					(int)st.st_uid, listdir);
-			if(setuid(st.st_uid) < 0) {
-				perror("setuid");
-				fprintf(stderr, "Continuing as uid %d\n",
-						(int)uid);
-			}
-		}
-	}
-
 	chstr[0] = address[0];
 	chstr[1] = '\0';
 	
@@ -667,61 +629,241 @@ int main(int argc, char **argv)
 		myfree(sublockname);
 		exit(EXIT_FAILURE);
 	}
-	subbed = is_subbed_in(subddirname, address);
-	listdelim = getlistdelim(listdir);
-	if(modstr == NULL)
-		submod = statctrl(listdir, "submod");
-	
-	if(subbed) {
-		if(subconfirm) {
-			close(subfilefd);
-			close(sublockfd);
-			unlink(sublockname);
-			myfree(sublockname);
-			generate_subconfirm(listdir, listaddr, listdelim,
-					    address, mlmmjsend, typesub);
-		} else {
-			if(submod) {
-				close(subfilefd);
-				close(sublockfd);
-				unlink(sublockname);
-				myfree(sublockname);
-				moderate_sub(listdir, listaddr, listdelim,
-					address, mlmmjsend, typesub);
-			}
-			lseek(subfilefd, 0L, SEEK_END);
-			len = strlen(address);
-			address[len] = '\n';
-			writen(subfilefd, address, len + 1);
-			address[len] = 0;
-			close(subfilefd);
-			close(sublockfd);
-			unlink(sublockname);
-		}
-	} else {
-		close(subfilefd);
-		myfree(subfilename);
-		close(sublockfd);
-		unlink(sublockname);
-		myfree(sublockname);
-
-		if(!nogensubscribed)
-			generate_subscribed(listdir, address, mlmmjsend);
-		
-		return EXIT_SUCCESS;
-	}
 
+	lseek(subfilefd, 0L, SEEK_END);
+	len = strlen(address);
+	address[len] = '\n';
+	writen(subfilefd, address, len + 1);
+	address[len] = 0;
+	close(subfilefd);
 	close(sublockfd);
 	unlink(sublockname);
 	myfree(sublockname);
+}
+
+int main(int argc, char **argv)
+{
+	char *listaddr, *listdelim, *listdir = NULL;
+	char *mlmmjsend, *mlmmjunsub, *bindir;
+	char *address = NULL, *lowcaseaddr, *modstr = NULL;
+	const char *flag = NULL;
+	int opt, subconfirm = 0, confirmsub = 0, notifysub;
+	int changeuid = 1, status, digest = 0, nomail = 0, both = 0;
+	int nogensubscribed = 0;
+	int force = 0, quiet = 0, i = 0;
+	enum subtype subbed;
+	struct stat st;
+	pid_t pid, childpid = 0;
+	uid_t uid;
+	enum subtype typesub = SUB_NORMAL;
+	enum subreason reasonsub = SUB_ADMIN;
+
+	CHECKFULLPATH(argv[0]);
+
+	log_set_name(argv[0]);
+
+	bindir = mydirname(argv[0]);
+	mlmmjsend = concatstr(2, bindir, "/mlmmj-send");
+	mlmmjunsub = concatstr(2, bindir, "/mlmmj-unsub");
+	myfree(bindir);
+
+	while ((opt = getopt(argc, argv, "hbcCdfm:nsVUL:a:qrR")) != -1) {
+		switch(opt) {
+		case 'a':
+			address = optarg;
+			break;
+		case 'b':
+			both = 1;
+			break;
+		case 'c':
+			confirmsub = 1;
+			break;
+		case 'C':
+			subconfirm = 1;
+			break;
+		case 'd':
+			digest = 1;
+			break;
+		case 'f':
+			force = 1;
+			break;
+		case 'h':
+			print_help(argv[0]);
+			break;
+		case 'L':
+			listdir = optarg;
+			break;
+		case 'm':
+			modstr = optarg;
+			break;
+		case 'n':
+			nomail = 1;
+			break;
+		case 'q':
+			quiet = 1;
+			break;
+		case 'r':
+			reasonsub = SUB_REQUEST;
+			break;
+		case 'R':
+			reasonsub = SUB_CONFIRM;
+			break;
+		case 's':
+			nogensubscribed = 1;
+			break;
+		case 'U':
+			changeuid = 0;
+			break;
+		case 'V':
+			print_version(argv[0]);
+			exit(0);
+		}
+	}
+
+	if(listdir == NULL) {
+		fprintf(stderr, "You have to specify -L\n");
+		fprintf(stderr, "%s -h for help\n", argv[0]);
+		exit(EXIT_FAILURE);
+	}
+	
+	if(address == NULL && modstr == NULL) {
+		fprintf(stderr, "You have to specify -a or -m\n");
+		fprintf(stderr, "%s -h for help\n", argv[0]);
+		exit(EXIT_FAILURE);
+	}
+
+	if(both + digest + nomail > 1) {
+		fprintf(stderr, "Specify at most one of -b, -d and -n\n");
+		fprintf(stderr, "%s -h for help\n", argv[0]);
+		exit(EXIT_FAILURE);
+	}
+
+	if(digest)
+		typesub = SUB_DIGEST;
+	if(nomail)
+		typesub = SUB_NOMAIL;
+	if(both)
+		typesub = SUB_BOTH;
+
+	if(reasonsub == SUB_CONFIRM && subconfirm) {
+		fprintf(stderr, "Cannot specify both -C and -R\n");
+		fprintf(stderr, "%s -h for help\n", argv[0]);
+		exit(EXIT_FAILURE);
+	}
+
+	if(modstr) {
+		getaddrandtype(listdir, modstr, &address, &typesub);
+		reasonsub = SUB_PERMIT;
+	}
+
+	if(strchr(address, '@') == NULL) {
+		log_error(LOG_ARGS, "No '@' sign in '%s', not subscribing",
+				address);
+		exit(EXIT_SUCCESS);
+	}
+
+	/* Make the address lowercase */
+	lowcaseaddr = mystrdup(address);
+	i = 0;
+	while(lowcaseaddr[i]) {
+		lowcaseaddr[i] = tolower(lowcaseaddr[i]);
+		i++;
+	}
+	address = lowcaseaddr;
+
+	/* get the list address */
+	listaddr = getlistaddr(listdir);
+	if(strncasecmp(listaddr, address, strlen(listaddr)) == 0) {
+		printf("Cannot subscribe the list address to the list\n");
+		exit(EXIT_SUCCESS);  /* XXX is this success? */
+	}
+
+	if(changeuid) {
+		uid = getuid();
+		if(!uid && stat(listdir, &st) == 0) {
+			printf("Changing to uid %d, owner of %s.\n",
+					(int)st.st_uid, listdir);
+			if(setuid(st.st_uid) < 0) {
+				perror("setuid");
+				fprintf(stderr, "Continuing as uid %d\n",
+						(int)uid);
+			}
+		}
+	}
+
+	subbed = is_subbed(listdir, address, 1);
+
+	if(subbed == typesub) {
+		if(!nogensubscribed)
+			generate_subscribed(listdir, address, mlmmjsend,
+					typesub);
+		return EXIT_SUCCESS;
+	} else if(subbed != SUB_NONE) {
+		reasonsub = SUB_SWITCH;
+		/* If we want to subscribe to both, we can just subscribe the
+		 * missing version, so don't unsub. */
+		if (!(typesub == SUB_BOTH &&
+				subbed != SUB_NOMAIL)) {
+			childpid = fork();
+			if(childpid < 0) {
+				log_error(LOG_ARGS, "Could not fork; "
+				"not unsubscribed from current version");
+			}
+			if (childpid == 0) {
+				if (subbed == SUB_BOTH) {
+					if (typesub == SUB_NORMAL) flag = "-d";
+					if (typesub == SUB_DIGEST) flag = "-N";
+				}
+				execlp(mlmmjunsub, mlmmjunsub,
+						"-L", listdir, "-q",
+						"-a", address, flag,
+						(char *)NULL);
+				log_error(LOG_ARGS, "execlp() of '%s' failed",
+						mlmmjunsub);
+				exit(EXIT_FAILURE);
+			}
+		}
+	}
+
+	if(childpid > 0) {
+		do /* Parent waits for the child */
+			pid = waitpid(childpid, &status, 0);
+		while(pid == -1 && errno == EINTR);
+	}
+
+	listdelim = getlistdelim(listdir);
+
+	if(subbed == SUB_NONE && subconfirm)
+			generate_subconfirm(listdir, listaddr, listdelim,
+			address, mlmmjsend, typesub, reasonsub);
+
+	if(modstr == NULL && subbed == SUB_NONE && !force &&
+			statctrl(listdir, "submod")) {
+		moderate_sub(listdir, listaddr, listdelim,
+				address, mlmmjsend, typesub, reasonsub);
+	}
+
+	if (typesub == SUB_BOTH) {
+		if (subbed != SUB_NORMAL) {
+			subscribe_type(listdir, listaddr, listdelim, address,
+					mlmmjsend, SUB_NORMAL, reasonsub);
+		}
+		if (subbed != SUB_DIGEST) {
+			subscribe_type(listdir, listaddr, listdelim, address,
+					mlmmjsend, SUB_DIGEST, reasonsub);
+		}
+	} else if (!(subbed == SUB_BOTH && typesub != SUB_NOMAIL)) {
+		subscribe_type(listdir, listaddr, listdelim, address,
+				mlmmjsend, typesub, reasonsub);
+	}
 
 	if(confirmsub) {
 		childpid = fork();
 
 		if(childpid < 0) {
-			log_error(LOG_ARGS, "Could not fork");
+			log_error(LOG_ARGS, "Could not fork; owner not notified");
 			confirm_sub(listdir, listaddr, listdelim, address,
-					mlmmjsend, typesub);
+					mlmmjsend, typesub, reasonsub);
 		}
 		
 		if(childpid > 0) {
@@ -733,16 +875,18 @@ int main(int argc, char **argv)
 		/* child confirms subscription */
 		if(childpid == 0)
 			confirm_sub(listdir, listaddr, listdelim, address,
-					mlmmjsend, typesub);
+					mlmmjsend, typesub, reasonsub);
 	}
 
-	notifysub = statctrl(listdir, "notifysub");
+	notifysub = !quiet && reasonsub != SUB_SWITCH &&
+			statctrl(listdir, "notifysub");
 
 	/* Notify list owner about subscription */
 	if (notifysub)
 		notify_sub(listdir, listaddr, listdelim, address, mlmmjsend,
-				typesub);
+				typesub, reasonsub);
 
+	myfree(address);
 	myfree(listaddr);
 	myfree(listdelim);
 
