@@ -1098,7 +1098,7 @@ char *get_processed_text_line(text *txt, int headers,
 	char *tmp;
 	char *prev = NULL;
 	int len, width, i;
-	int processedlen, processedwidth;
+	int processedlen = 0, processedwidth = 0;
 	int wrapindentlen = -1;
 	int incision, linebreak, linebreakwidth;
 	int directive, inhibitbreak;
@@ -1700,7 +1700,7 @@ char *prepstdreply(text *txt, const char *listdir,
 			if (strncasecmp(line, "Subject:", len) == 0) {
 				tmp = unistr_utf8_to_header(tmp);
 				myfree(line);
-				line = concatstr(2, "Subject:", tmp);
+				line = concatstr(2, "Subject: ", tmp);
 				myfree(tmp);
 			}
 			len = strlen(line);
